from typing import Optional

from ..guard.config import Guard
from ..models.scanresponse import Extraction
from ..utils.logger import get_default_logger
from .parser import ResponseParser
from .result import GuardMatch, ResponseMatch

logger = get_default_logger()

class ResponseEvaluator:
    """
    This ResponseEvaluator determines if conditions are met based on thresholds.
    """

    def __init__(self):
        self._parser = ResponseParser()  # Use the existing ResponseParser
        self._response = None

    def evaluate(
        self,
        response_extraction: Extraction,
        guard: Guard,
        match_name: Optional[str] = None
    ) -> GuardMatch:
        """
        Evaluates a check condition using a Guard object.

        Args:
            response_extraction: The scan response extraction
            guard: The guard to eval with the response
            match_name: The match match for the guard

        Returns:
            GuardMatch with MATCH.YES if condition met, MATCH>NO if not met
        """
        try:
            result = self._parser.get_value(response_extraction, guard, match_name)
            # Handle different return types
            # PII and keyword
            match_count = None
            if isinstance(result, tuple) and len(result) == 3:  # (bool, float, int)
                exists, value, match_count = result
            # exploit, topic, classification, language
            elif isinstance(result, tuple) and len(result) == 2:  # (bool, float)
                exists, value = result
            # secrets and modality
            elif isinstance(result, bool):  # bool only
                exists, value = result, 1.0
            else:
                raise ValueError("Unexpected return type from get_value")

            if not exists:
                return GuardMatch(
                    response_match=ResponseMatch.NO,
                    guard_name=guard.name,
                    threshold=str(guard.threshold),
                    actual_value=value
                )
            # Use ThresholdHelper for comparison
            comparison_result = guard.threshold.compare(value)

            return GuardMatch(
                response_match=ResponseMatch.YES if comparison_result else ResponseMatch.NO,
                guard_name=guard.name,
                threshold=str(guard.threshold),
                actual_value=value,
                match_count=match_count if match_count else 0
            )
        except Exception as e:
            logger.debug("Error in check evaluation: %s", str(e))
            raise



from enum import Enum, auto
from typing import Dict

from acuvity.guard.constants import GuardName
from acuvity.models.textualdetection import TextualdetectionType

# Default action for guards
DEFAULT_ACTION = "deny"

guardname_analyzer_id_map: Dict[GuardName, str] = {
    # Exploit guards
    GuardName.PROMPT_INJECTION: 'en-text-prompt_injection-detector',
    GuardName.JAIL_BREAK: 'en-text-jailbreak-detector',
    GuardName.MALICIOUS_URL: 'url-malicious-detector',

    # Topic guards with prefixes
    GuardName.TOXICITY: 'en-text-toxicity-detector',
    GuardName.BIAS: 'en-text-bias-detector',
    GuardName.HARMFUL_CONTENT: 'en-text-harmful-content-detector',

    # Other guards
    GuardName.LANGUAGE: 'text-language-classifier',
    GuardName.PII_DETECTOR: 'en-text-ner-detector',
    GuardName.SECRETS_DETECTOR:'text-pattern-detector',
    GuardName.KEYWORD_DETECTOR: 'text-keyword-detector',
    GuardName.MODALITY: 'modality-detector',
}

class GuardType(Enum):
    """Types of guards and their corresponding sections in Extraction."""
    EXPLOIT = auto()         # Direct access in exploits
    TOPIC = auto()           # Needs prefix in topics
    LANGUAGE = auto()        # Can be direct or match-based
    PII = auto()             # Both direct values and textual detection counts
    SECRETS = auto()         # In secrets section
    KEYWORD = auto()         # In textual detections with count
    MODALITY = auto()        # Special handling for modalities list

# Define mappings
GUARD_TYPES = {
    # Exploit guards
    GuardName.PROMPT_INJECTION: GuardType.EXPLOIT,
    GuardName.JAIL_BREAK: GuardType.EXPLOIT,
    GuardName.MALICIOUS_URL: GuardType.EXPLOIT,

    # Topic guards with prefixes
    GuardName.TOXICITY: GuardType.TOPIC,
    GuardName.BIAS: GuardType.TOPIC,
    GuardName.HARMFUL_CONTENT: GuardType.TOPIC,

    # Other guards
    GuardName.LANGUAGE: GuardType.LANGUAGE,
    GuardName.PII_DETECTOR: GuardType.PII,
    GuardName.SECRETS_DETECTOR: GuardType.SECRETS,
    GuardName.KEYWORD_DETECTOR: GuardType.KEYWORD,
    GuardName.MODALITY: GuardType.MODALITY
}

# Topic prefixes mapping
TOPIC_PREFIXES = {
    GuardName.TOXICITY: 'content/toxic',
    GuardName.BIAS: 'content/bias',
    GuardName.HARMFUL_CONTENT: 'content/harmful',
}

DETECTIONTYPE_MAP = {
    TextualdetectionType.KEYWORD : "keywords",
    TextualdetectionType.PII: "pi_is",
    TextualdetectionType.SECRET : "secrets"
}

GUARDNAME_TO_DETECTIONTYPE = {
    GuardName.KEYWORD_DETECTOR : TextualdetectionType.KEYWORD,
    GuardName.SECRETS_DETECTOR: TextualdetectionType.SECRET,
    GuardName.PII_DETECTOR: TextualdetectionType.PII,
    GuardName.LANGUAGE: "language",
}



from typing import Optional, Tuple, Union

from acuvity.guard.config import Guard
from acuvity.guard.constants import GuardName
from acuvity.models.extraction import Extraction
from acuvity.response.constants import (
    DETECTIONTYPE_MAP,
    GUARD_TYPES,
    GUARDNAME_TO_DETECTIONTYPE,
    TOPIC_PREFIXES,
    GuardType,
)
from acuvity.response.errors import ResponseValidationError


class ResponseParser:
    """Parser for accessing values in Extraction response based on guard types."""

    def get_value(
        self,
        extraction: Extraction,
        guard: Guard,
        match_name: Optional[str] = None
    ) -> Union[bool, Tuple[bool, float], Tuple[bool, float, int]]:
        """Get value from extraction based on guard type."""
        guard_type = GUARD_TYPES.get(guard.name)
        if not guard_type:
            raise ResponseValidationError(f"Unknown guard type: {guard.name}")

        value_getters = {
            GuardType.EXPLOIT: self._get_guard_value,
            GuardType.TOPIC: self._get_guard_value,
            GuardType.LANGUAGE: self._get_language_value,
            GuardType.MODALITY: self._get_modality_value,
            GuardType.PII: self.get_text_detections,
            GuardType.SECRETS: self.get_text_detections,
            GuardType.KEYWORD: self.get_text_detections,
        }

        getter = value_getters.get(guard_type)
        if not getter:
            raise ResponseValidationError(f"No handler for guard type: {guard_type}")

        try:
            return getter(extraction, guard, match_name)
        except Exception as e:
            raise ResponseValidationError(f"Error getting value for {guard.name}: {str(e)}") from e

    def _get_guard_value(
        self,
        extraction: Extraction,
        guard: Guard,
        _: Optional[str]
    ) -> tuple[bool, float]:
        """Get value from topics section with prefix handling."""

        prefix = TOPIC_PREFIXES.get(guard.name)
        if not prefix:
            if not extraction.exploits:
                return False , 0.0
            value = extraction.exploits.get(str(guard.name))
            if value is None:
                return False, 0
            return True, float(value)
        else:
            if not extraction.topics:
                return False, 0
            value = extraction.topics.get(prefix)
            if value is not None:
                return True, float(value)
            return False, 0.0


    def _get_language_value(
        self,
        extraction: Extraction,
        _: Guard,
        match_name: Optional[str]
    ) -> tuple[bool, float]:
        """Get value from languages section."""
        if not extraction.languages:
            return False, 0

        if match_name:
            value = extraction.languages.get(match_name)
        else:
            return len(extraction.languages) > 0 , 1.0

        if value is None:
            return False, 0
        return True, float(value)


    def get_text_detections(
            self,
            extraction: Extraction,
            guard: Guard,
            match_name: Optional[str]
    )-> tuple[bool, float, int]:

        detection_type = GUARDNAME_TO_DETECTIONTYPE.get(guard.name)
        if not detection_type:
            raise ResponseValidationError(f"No matching detection type for guard: {guard.name}")

        # Get the field name for this detection type
        field_name = DETECTIONTYPE_MAP.get(detection_type)
        if not field_name:
            raise ResponseValidationError(f"No matching field for detection type: {detection_type}")

        # Get the relevant match_keys dictionary using getattr
        match_keys = getattr(extraction, field_name, {}) or {}
        detections = extraction.detections or []

        if not match_keys and not detections:
            return False, 0, 0

        if match_name:
            # Count occurrences in textual detections
            if not detections:
                return False, 0, 0
            text_matches = []
            text_matches = [
                d.score for d in detections
                if d.type == detection_type and d.name == match_name and d.score is not None  and guard.threshold.compare(d.score)
            ]

            count = len(text_matches)
            # If no textual detections, check `match_keys` for the match
            if count == 0 and match_keys and match_name in match_keys:
                return True, match_keys[match_name], 1

            if count == 0:
                return False, 0, 0

            score = max(text_matches)
            return True, score, count

        # Return all text match values if no match_name is provided
        exists = bool(match_keys)
        count = len(match_keys) if match_keys else 0
        return exists, 1.0 if exists else 0.0, count

    def _get_modality_value(
        self,
        extraction: Extraction,
        _: GuardName,
        match_name: Optional[str] = None
    ) -> bool:
        if not extraction.modalities:
            return False  # No modalities at all

        if match_name:
            # Check for specific modality
            return any(m.group == match_name for m in extraction.modalities)

        # Check if any modality exists
        return len(extraction.modalities) > 0



from typing import List, Optional

from acuvity.guard.config import Guard, GuardConfig
from acuvity.models.scanresponse import Extraction, Scanresponse
from acuvity.response.evaluator import ResponseEvaluator
from acuvity.response.result import GuardMatch, Matches, ResponseMatch
from acuvity.utils.logger import get_default_logger

logger = get_default_logger()

class ResponseProcessor:
    """Handles processing of guard configurations."""

    def __init__(self, response: Scanresponse, guard_config: GuardConfig):
        self._evaluator = ResponseEvaluator()
        self.guard_config = guard_config
        self._response = response

    def process_guard_check(
        self,
        guard: Guard,
        extraction: Extraction,
        match_name: Optional[str] = None
    ) -> GuardMatch:
        """Process a single guard check with action consideration."""
        try:
            # Get raw evaluation
            return self._evaluator.evaluate(extraction, guard, match_name)
        except Exception as e:
            logger.debug("Error processing guard %s ", {guard.name})
            raise e

    def _process_simple_guard(self, guard: Guard, extraction: Extraction) -> GuardMatch:
        """Process a simple guard (no matches)."""
        return self.process_guard_check(guard, extraction)

    def _process_match_guard(self, guard: Guard, extraction: Extraction) -> GuardMatch:
        """Process a guard with matches."""
        result_match = ResponseMatch.NO
        match_counter = 0
        for match_name, match_name_guard in guard.matches.items():
            result = self.process_guard_check(
                guard,
                extraction,
                match_name
            )
            # increment the match_counter only if eval is YES and it crosess the individual count_threshold .
            if result.response_match == ResponseMatch.YES and result.match_count >= match_name_guard.count_threshold:
                match_counter += result.match_count
            # if any one match, then flagged or if threshold given then check if greater.
            if match_counter >= guard.count_threshold:
                result_match = ResponseMatch.YES

        logger.debug(f"match guard {guard.name}, check {result_match}, total match {match_counter}, guard threshold {guard.count_threshold}")
        return GuardMatch(
                    response_match=result_match,
                    guard_name=guard.name,
                    threshold=str(guard.threshold),
                    actual_value=1.0,
                    match_count=match_counter
                )

    def matches(self) -> list[Matches]:
        """Process the complete guard configuration."""
        all_matches : list[Matches] = []
        try:
            if self._response.extractions is None:
                raise ValueError("response doesn't contain extractions")

            for ext in self._response.extractions:
                if ext.data is None:
                    continue
                matched_checks = []
                all_checks = []
                for guard in self.guard_config.simple_guards:
                    result = self._process_simple_guard(guard, ext)
                    if result.response_match == ResponseMatch.YES:
                        matched_checks.append(result)
                    all_checks.append(result)

                for guard in self.guard_config.match_guards:
                    results = self._process_match_guard(guard, ext)
                    if results.response_match == ResponseMatch.YES:
                        matched_checks.append(results)
                    all_checks.append(results)

                single_match =  Matches(
                    input_data=ext.data,
                    response_match=ResponseMatch.YES if matched_checks else ResponseMatch.NO,
                    matched_checks=matched_checks,
                    all_checks=all_checks
                )
                all_matches.append(single_match)
            return all_matches

        except Exception as e:
            logger.debug("Error processing guard config: %s",{str(e)})
            raise ValueError(f"Failed to process guard configuration: {str(e)}") from e
